/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.datamodel
 * File:    LinkRep.java
 *
 * Copyright (C) 2010-2011 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.datamodel;

import java.util.Collection;

import de.fhhannover.inform.iron.mapserver.datamodel.graph.Identifier;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.Link;
import de.fhhannover.inform.iron.mapserver.utils.LinkedMap;

/**
 * LinkRep
 * 
 * @author awe, vp
 * @version 0.1
 */

/*
 * created: 19.11.09
 * changes:
 * 	22.11.09 vp - Added: linkrepository, getInstance()
 *  26.11.09 vp - empty methods implemented
 *  01.12.09 vp - changed HashMap<Integer,Link> to HashMap<Integer,LinkedList<Link>> for all Methods
 *  12.12.09 vp - addLink() changed to private and return link, getOrCreateLink() changed 
 *  22.12.09 vp - removeLink bug fixed
 *  03.01.10 vp - new data structure
 *  11.02.10 vp - the visibility of the classes and methods is changed
 *  30.04.10 aw - remove singletons!
 */
class LinkRep {
 
	private LinkedMap<Link> links;
	private IdentifierRep identifierRep;
	
	LinkRep(IdentifierRep iRep) {
		identifierRep = iRep;
		links = new LinkedMap<Link>();
	}
	
	/**
	 * If link does not exist create one
	 * @param ident1
	 * @param ident2
	 */
	private Link addLink(Identifier ident1, Identifier ident2) {
			Identifier tempIdent1 = identifierRep.getOrCreateIdentifier(ident1);
			Identifier tempIdent2 = identifierRep.getOrCreateIdentifier(ident2);
			
			Link newLink = new Link(tempIdent1, tempIdent2);
			tempIdent1.addLink(newLink);
			tempIdent2.addLink(newLink);
			links.put(newLink.hashCode(), newLink);
			return newLink;
	}
	
	/**
	 * Search link in repository if link does not found create one
	 * @param id1
	 * @param id2
	 * @return Link
	 */
	Link getOrCreateLink(Identifier id1, Identifier id2) {
		if (id1 == null || id2 == null) {
			return null;
		}
		
		Link tempLink = getLink(id1,id2);
		if (tempLink == null) {
			tempLink = addLink(id1,id2);
		}
		return tempLink;
	}
	
	/**
	 * Search link in repository and get it
	 * @param ident1
	 * @param ident2
	 * @return if link exist get link or if link does not exist get null
	 */
	Link getLink(Identifier ident1, Identifier ident2) {
		if (ident1 == null || ident2 == null) {
			return null;
		}
		
		Link tempLink = new Link(ident1, ident2);
		return links.getElement(tempLink.hashCode(), tempLink);
	}
	
	/**
	 * Remove link from link repository
	 * @param link
	 */
	void removeLink(Link link) {
		Link tempLink = getLink(link.getIdent1(),link.getIdent2());
		if (tempLink != null) {
			tempLink.getIdent1().removeLink(tempLink);
			tempLink.getIdent2().removeLink(tempLink);
			links.remove(link.hashCode(), link);
		}
	}
	
	String graphvizEdges() {
		StringBuffer sb = new StringBuffer();
		Collection<Link> vlist = links.values();
		
		for (Link l : vlist) {
			sb.append("\t");
			sb.append((long)((long)l.getIdent1().hashCode() * (long)l.getIdent1().hashCode()));
			sb.append(" -- ");
			sb.append((long)((long)l.getIdent2().hashCode() * (long)l.getIdent2().hashCode()));
			sb.append("\n");
		}
		return sb.toString();
	}
}
 

