/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.datamodel
 * File:    SearchResultBuilderImpl.java
 *
 * Copyright (C) 2010-2011 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.datamodel;

import java.util.LinkedList;
import java.util.List;

import de.fhhannover.inform.iron.mapserver.datamodel.graph.MetadataContainer;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.meta.Metadata;
import de.fhhannover.inform.iron.mapserver.messages.SearchResult;

/**
 * Implementation of the interface {@link SearchResultBuilder}
 * 
 * @author aw
 * @version 0.1
 *
 */

/*
 * created: 01.05.10 aw
 * changes:
 *  01.05.10 aw - ...
 * 
 */
public class SearchResultBuilderImpl extends SearchResultBuilder {
	
	
	private SearchResult searchResult;
	
	public SearchResultBuilderImpl(SearchResult sRes) {
		searchResult = sRes;
	}
	
	@Override
	public void addMetadata(MetadataContainer mc, Metadata m) {
		searchResult.addMetadata(mc, m);
	}

	@Override
	public void addMetadataContainer(MetadataContainer mc) {
		searchResult.addMetadataContainer(mc);
	}

	
	@Override
	public SearchResult getResult() {
		return searchResult;
	}

	
	@Override
	public void filter(Filter resultFilter) {
		List<Metadata> toRemove = new LinkedList<Metadata>();
		List<Metadata> mcMetadata = null;
		
		for (MetadataContainer mc : searchResult.getIdentifiers()) {
			mcMetadata = searchResult.getMetadataFor(mc);
			for (Metadata m : mcMetadata) {
				if (!m.matchesFilter(resultFilter)) {
					toRemove.add(m);
				}
			}
			mcMetadata.removeAll(toRemove);
		}
		
		for (MetadataContainer mc : searchResult.getLinks()) {
			mcMetadata = searchResult.getMetadataFor(mc);
			for (Metadata m : mcMetadata) {
				if (!m.matchesFilter(resultFilter)) {
					toRemove.add(m);
				}
			}
			mcMetadata.removeAll(toRemove);
		}
	}

	@Override
	public int getByteCount() {
		return searchResult.getByteCount();
	}

}
